package cz.vsb.fei;

import cz.vsb.fei.java2.lab01text2asciiart.ConversionException;
import cz.vsb.fei.java2.lab01text2asciiart.Text2AsciiArt;
import lombok.extern.log4j.Log4j2;

import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.Scanner;

/**
 * Class <b>App</b> - main class
 * 
 * @author Java I
 */
@Log4j2
public class App {

	public static void main(String[] args) {
		log.info("Launching Java application.");
		log.info("All args: {}", () -> Arrays.toString(args));
		List<String> argsList = Arrays.asList(args);
		String text = null;
		String fontName = null;
		boolean cliFlag = false;
		Text2AsciiArt asciiArt = new Text2AsciiArt();
		if (argsList.contains("-help")) {
			printHelp();
		}
		if (argsList.contains("-fonts")) {
			System.out.println("Available font names:");
			asciiArt.getAllfontNames().forEach(System.out::println);
		}
		fontName = getArgValue(argsList, "-font");
		text = getArgValue(argsList, "-text");
		if (text == null && argsList.contains("-cli")) {
			cliFlag = true;
		}
		if (argsList.contains("-cli")) {
			cliFlag = true;
		}
		if (argsList.contains("-demo")) {
			for (String font : asciiArt.getAllfontNames()) {
				System.out.println("Font name: " + font);
				System.out.println("======================================================== Font Example ==================================================================");
				try {
					System.out.println(asciiArt.convert("Hello: ABCD - abcd _ 1234 +-*/", font));
				} catch (ConversionException e) {
					e.printStackTrace();
				}
				System.out.println("========================================================================================================================================");
			}
		}
		if(argsList.isEmpty()) {
			printHelp();
			System.out.println("No parameter supplied! Using default text and random font.");
			List<String> fontNames = asciiArt.getAllfontNames();
			fontName = fontNames.get(new Random().nextInt(fontNames.size()));
			System.out.println("Choosed font name: " + fontName);
			text = "Hello user " + System.getProperty("user.name");
		}
		if (cliFlag) {
			System.out.println("Type text to convert:");
			try (Scanner scanner = new Scanner(System.in)) {
				text = scanner.nextLine();
			}
		}
		if(text == null || text.isEmpty() || text.isBlank()){
			text = "Empty text supplied!";
		}
		try {
			if (fontName != null) {
				System.out.println(asciiArt.convert(text, fontName));
			} else {
				System.out.println(asciiArt.convert(text));
			}
		} catch (ConversionException e) {
			e.printStackTrace();
		}

	}

	private static String getArgValue(List<String> argsList, String argName) {
		if (argsList.contains(argName)) {
			int fontIndex = argsList.indexOf(argName) + 1;
			if (fontIndex < argsList.size()) {
				return argsList.get(fontIndex);
			} else {
				System.out.println("No font name supplied! Use default font.");
			}
		}
		return null;
	}

	private static void printHelp() {
		System.out.println("Use params:   -text custom_text [-font font_name]");
		System.out.println("              -cli [-font font_name]");
		System.out.println("              -fonts");
		System.out.println("              -help");
		System.out.println("              -demo");
	}

}