package jez04.structure.test;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URISyntaxException;

import org.junit.jupiter.api.Test;

class ClassStructureTest {

	StructureHelper helper = StructureHelper.getInstance();

	@Test
	void outputStringTest() throws InvocationTargetException, IllegalAccessException {
		helper.classExist("App", false);
		Class<?> app = helper.getClass("App", false);
		Method main = helper.getMethod(app, "main", void.class, String[].class);
		PrintStream original = System.out;
		ByteArrayOutputStream dataStream = new ByteArrayOutputStream();
		System.setOut(new PrintStream(dataStream));
		try {
			main.invoke(null, new Object[][]{new String[]{"-text", "Test Text"}});
			assertNotEquals(0, dataStream.toByteArray().length, "No data written to output consola");
			assertTrue(new String(dataStream.toByteArray()).split("\n").length > 5, "less then 5 lines writen to output consola");
        } finally {
			System.setOut(original);
			try {
				System.out.write(dataStream.toByteArray());
				dataStream.close();
			} catch (IOException e) {
				/* ignore it */}
		}
	}

	@Test
	void inputStringTest() throws InvocationTargetException, IllegalAccessException {
		helper.classExist("App", false);
		Class<?> app = helper.getClass("App", false);
		Method main = helper.getMethod(app, "main", void.class, String[].class);
		PrintStream original = System.out;
		ByteArrayOutputStream dataStream = new ByteArrayOutputStream();
		System.setOut(new PrintStream(dataStream));
		System.setIn(new ByteArrayInputStream("----------\na\n\n\n\n\n".getBytes()));
		try {
			System.setIn(new ByteArrayInputStream("----------\n".getBytes()));
			main.invoke(null, new Object[][]{new String[]{"-cli"}});
			System.setIn(new ByteArrayInputStream("----------\na\n\n\n\n\n".getBytes()));
			main.invoke(null, new Object[][]{new String[]{}});
			assertNotEquals(0, dataStream.toByteArray().length, "No data written to output consola");
			long count = new String(dataStream.toByteArray()).chars().filter(c -> c=='_').count();
			assertTrue(count > 90, "String from consola (input stream) is not used");
		} finally {
			System.setOut(original);
			try {
				System.out.write(dataStream.toByteArray());
				dataStream.close();
			} catch (IOException e) {
				/* ignore it */
			}
		}
	}

	@Test
	void mavenAssemblyPluginTest() throws URISyntaxException, IOException {
		String pom = helper.getProjectFile("pom.xml");
		assertTrue(pom.contains("maven-assembly-plugin"), "No jlink used");
		assertTrue(pom.contains("<manifest>"), "No jlink used");
	}

	@Test
	void mavenDependencyPluginTest() throws URISyntaxException, IOException {
		String pom = helper.getProjectFile("pom.xml");
		assertTrue(pom.contains("maven-dependency-plugin"), "No jlink used");
		assertTrue(pom.contains("copy-dependencies"), "No jlink used");
		assertTrue(pom.contains("${project.build.directory}"), "No jlink used");
	}
	@Test
	void mavenJarPluginTest() throws URISyntaxException, IOException {
		String pom = helper.getProjectFile("pom.xml");
		assertTrue(pom.contains("maven-jar-plugin"), "No jlink used");
		assertTrue(pom.contains("manifestEntries"), "No jlink used");
		assertTrue(pom.contains("addClasspath"), "No jlink used");
		assertTrue(pom.contains("classpathPrefix"), "No jlink used");
		assertFalse(pom.contains("--must be changed"), "Contains --must be changed");
}

	@Test
	void mavenJlinkTest() throws URISyntaxException, IOException {
		String pom = helper.getProjectFile("pom.xml");
		assertTrue(pom.contains("maven-jlink-plugin"), "No jlink used");
		assertTrue(pom.contains("create-jlink-image"), "No jlink used");
	}

	@Test
	void mavenVsbLibTest() throws URISyntaxException, IOException {
		String pom = helper.getProjectFile("pom.xml");
		assertTrue(pom.contains("lab01-text2asciiart"), "No lab01-text2asciiart used");
		assertTrue(pom.contains("repositories"), "No jlink used");
		assertTrue(pom.contains("https://artifactory.cs.vsb.cz/repository"), "No jlink used");
	}

}
