package jez04.structure.test;

import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.FileVisitResult;
import java.nio.file.FileVisitor;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Pattern;

import org.hamcrest.Description;

public class ProjectContains extends StructureMatcher<String> {
	private String regexp;
	private boolean caseInsensitive;
	private boolean fileFound;
	private int count = 1;
	private List<Path> foundFiles;

	public ProjectContains(String regexp, boolean caseInsensitive) {
		this.regexp = regexp;
		this.caseInsensitive = caseInsensitive;
	}

	public ProjectContains count(int count) {
		this.count = count;
		return this;
	}

	@Override
	public boolean matches(Object actual) {
		fileFound = true;
		foundFiles = new LinkedList<>();
		Pattern p;
		if (caseInsensitive) {
			p = Pattern.compile(regexp, Pattern.CASE_INSENSITIVE);
		} else {
			p = Pattern.compile(regexp);
		}
		try {
			URL myClassUrl = StructureHelper.class.getResource(this.getClass().getSimpleName() + ".class");
			Path classRoot = Paths.get(myClassUrl.toURI());
			while (!"test-classes".equals(classRoot.getFileName().toString())
					&& !"classes".equals(classRoot.getFileName().toString())) {
				classRoot = classRoot.getParent();
			}
			Path projectRoot = classRoot.getParent().getParent();
			Files.walkFileTree(projectRoot, new FileVisitor<Path>() {

				@Override
				public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
					return FileVisitResult.CONTINUE;
				}

				@Override
				public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
					if (p.matcher(file.getFileName().toString()).matches()) {
						foundFiles.add(file);
					}
					return FileVisitResult.CONTINUE;
				}

				@Override
				public FileVisitResult visitFileFailed(Path file, IOException exc) throws IOException {
					return FileVisitResult.CONTINUE;
				}

				@Override
				public FileVisitResult postVisitDirectory(Path dir, IOException exc) throws IOException {
					return FileVisitResult.CONTINUE;
				}
			});
			return foundFiles.size() >= count;
		} catch (URISyntaxException | IOException e) {
			fileFound = false;
			e.printStackTrace();
			return false;
		}
	}

	@Override
	public void describeTo(Description description) {
		description.appendText(String.format("Source code of class shoud contains regexp '%s'%s", regexp,
				caseInsensitive ? " in case insensitive mode" : ""));
	}

	@Override
	public void describeMismatch(Object item, Description description) {
		if (fileFound) {
			description.appendText(String.format("Project do not contains %d file match regexp: %s", count, regexp));
		} else {
			description.appendText(String.format("source code of class %s was not found"));
		}
	}

	public List<Path> getFoundFiles() {
		return foundFiles;
	}

}
